import argparse

import imp
import sys

from subprocess import CalledProcessError
from unittest import TestCase

from httplib2 import ServerNotFoundError
from mock import (Mock, patch)
from nose.tools import (assert_equals, assert_false, raises)

from hubugs import utils


class Colouriser(TestCase):
    @patch('sys.stdout')
    def test_is_not_a_tty_colour(self, stdout):
        stdout.isatty = Mock(return_value=False)
        assert_false(sys.stdout.isatty())
        assert_equals(utils.success('test'), 'test')
        assert_equals(utils.fail('test'), 'test')
        assert_equals(utils.warn('test'), 'test')

    @patch('sys.stdout')
    def test_colouriser(self, stdout):
        stdout.isatty = Mock(return_value=True)

        # This horrific-ness is here as the .isatty() test happens at import
        # time, so we need to re-import for tests.
        utils_file = open('hubugs/utils.py')
        new_utils = imp.load_module("new_utils", utils_file, utils_file.name,
                           (".py", utils_file.mode, imp.PY_SOURCE))

        assert_equals(new_utils.success('test'), '\x1b[32mtest\x1b[0m')
        assert_equals(new_utils.fail('test'), '\x1b[31mtest\x1b[0m')
        assert_equals(new_utils.warn('test'), '\x1b[33mtest\x1b[0m')
        del sys.modules['new_utils']


class ProjectAction(TestCase):
    def setUp(self):
        self.parser = utils.argh.ArghParser('test_parser')
        # Stub out ._print_message() to stop help messages being displayed on
        # stderr during tests.
        self.parser._print_message = Mock(return_value=True)
        self.namespace = argparse.Namespace()
        self.action = utils.ProjectAction([], '')

    @patch('hubugs.utils.get_github_api')
    @patch('hubugs.utils.get_git_config_val')
    def test_repo_name(self, get_git_config_val, get_github_api):
        get_github_api().repos.show = Mock(return_value=True)
        get_git_config_val.return_value = 'JNRowe'

        self.action(self.parser, self.namespace, 'misc-overlay')
        assert_equals(self.namespace.project, 'JNRowe/misc-overlay')

        self.action(self.parser, self.namespace, 'JNRowe/misc-overlay')
        assert_equals(self.namespace.project, 'JNRowe/misc-overlay')

        self.action(self.parser, self.namespace, 'ask/python-github2')
        assert_equals(self.namespace.project, 'ask/python-github2')

    @patch('hubugs.utils.get_github_api')
    @patch('hubugs.utils.get_git_config_val')
    @raises(SystemExit)
    def test_no_user(self, get_git_config_val, get_github_api):
        get_github_api().repos.show = Mock(return_value=True)
        get_git_config_val.return_value = None
        self.action(self.parser, self.namespace, 'misc-overlay')

    @patch('hubugs.utils.get_github_api')
    @patch('hubugs.utils.get_git_config_val')
    @raises(SystemExit)
    def test_repo_not_found(self, get_git_config_val, get_github_api):
        get_github_api().repos.show = \
            Mock(side_effect=RuntimeError('Repository not found'))
        get_git_config_val.return_value = None
        self.action(self.parser, self.namespace, 'JNRowe/never_exist')

    @patch('hubugs.utils.get_github_api')
    @patch('hubugs.utils.get_git_config_val')
    @raises(RuntimeError)
    def test_random_error(self, get_git_config_val, get_github_api):
        get_github_api().repos.show = \
            Mock(side_effect=RuntimeError('something went wrong'))
        get_git_config_val.return_value = None
        self.action(self.parser, self.namespace, 'JNRowe/misc-overlay')

    @patch('hubugs.utils.get_github_api')
    @patch('hubugs.utils.get_git_config_val')
    @raises(SystemExit)
    def test_httplib2_error(self, get_git_config_val, get_github_api):
        get_github_api().repos.show = Mock(side_effect=ServerNotFoundError())
        get_git_config_val.return_value = None
        self.action(self.parser, self.namespace, 'JNRowe/misc-overlay')

    @patch('hubugs.utils.get_github_api')
    @patch('hubugs.utils.get_git_config_val')
    @raises(SystemExit)
    def test_env_error(self, get_git_config_val, get_github_api):
        get_github_api().repos.show = \
            Mock(side_effect=EnvironmentError('config broken'))
        get_git_config_val.return_value = None
        self.action(self.parser, self.namespace, 'JNRowe/misc-overlay')


class GetGithubApi(TestCase):
    @staticmethod
    def fake_env(key, default):
        fake_data = {
            'GITHUB_USER': 'JNRowe',
            'GITHUB_TOKEN': 'xxx',
            'HOME': '/home/JNRowe',
            'XDG_CACHE_HOME': 'cache_dir',
        }
        return fake_data[key]

    @patch('os.getenv')
    @patch('os.mkdir')
    def test_api_builder(self, mkdir, getenv):
        mkdir.return_value = True
        getenv.side_effect = self.fake_env
        api = utils.get_github_api()
        assert_equals(api.request.username, 'JNRowe')
        assert_equals(api.request.api_token, 'xxx')
        assert_equals(api.request._http.cache.cache, 'cache_dir/hubugs')

    @patch('os.getenv')
    @raises(EnvironmentError)
    def test_invalid_auth(self, getenv):
        getenv.return_value = None
        utils.get_github_api()

    @patch('os.getenv')
    @patch('os.mkdir')
    @patch('inspect.getargspec')
    def test_no_cache_support(self, getargspec, mkdir, getenv):
        getargspec().args = ['no_entry_cache_in_list', ]
        mkdir.return_value = True
        getenv.side_effect = self.fake_env
        api = utils.get_github_api()
        assert_equals(api.request._http.cache, None)


class GetGitConfigVal(TestCase):
    @patch('hubugs.utils.check_output')
    def test_valid_key(self, check_output):
        check_output.return_value = 'JNRowe'
        assert_equals(utils.get_git_config_val('github.user'), 'JNRowe')

    @patch('hubugs.utils.check_output')
    def test_invalid_key(self, check_output):
        check_output.return_value = ''
        assert_equals(utils.get_git_config_val('no_such_key'), '')

    @patch('hubugs.utils.check_output')
    def test_command_error(self, check_output):
        check_output.side_effect = CalledProcessError('255', 'cmd')
        assert_equals(utils.get_git_config_val('github.user'), None)


class GetEditor(TestCase):
    @patch('os.getenv')
    def test_git_editor_envvar(self, getenv):
        getenv.return_value = 'custom git editor'
        assert_equals(utils.get_editor(), 'custom git editor')

    @patch('hubugs.utils.get_git_config_val')
    @patch('os.getenv')
    def test_git_editor_config(self, getenv, get_git_config_val):
        getenv.return_value = None
        get_git_config_val.return_value = 'custom config editor'
        assert_equals(utils.get_editor(), 'custom config editor')

    @patch('hubugs.utils.get_git_config_val')
    @patch('os.getenv')
    def test_editor_environment_visual(self, getenv, get_git_config_val):
        def fake_env(key, default=None):
            return {'VISUAL': 'visual'}.get(key)
        getenv.side_effect = fake_env
        get_git_config_val.return_value = None
        assert_equals(utils.get_editor(), 'visual')

    @patch('hubugs.utils.get_git_config_val')
    @patch('os.getenv')
    def test_editor_environment_editor(self, getenv, get_git_config_val):
        def fake_env(key, default=None):
            return {'EDITOR': 'editor'}.get(key, default)
        getenv.side_effect = fake_env
        get_git_config_val.return_value = None
        assert_equals(utils.get_editor(), 'editor')

    @patch('hubugs.utils.get_git_config_val')
    @patch('os.getenv')
    def test_editor_default(self, getenv, get_git_config_val):
        def fake_env(key, default=None):
            return default
        getenv.side_effect = fake_env
        get_git_config_val.return_value = None
        assert_equals(utils.get_editor(), 'vi')


class GetRepo(TestCase):
    @patch('hubugs.utils.get_git_config_val')
    def test_ssh_url(self, get_git_config_val):
        get_git_config_val.return_value = \
            'git@github.com:JNRowe/misc-overlay.git'
        assert_equals(utils.get_repo(), 'JNRowe/misc-overlay')

    @patch('hubugs.utils.get_git_config_val')
    def test_ssh_url_no_suffix(self, get_git_config_val):
        get_git_config_val.return_value = \
            'git@github.com:JNRowe/misc-overlay'
        assert_equals(utils.get_repo(), 'JNRowe/misc-overlay')

    @patch('hubugs.utils.get_git_config_val')
    def test_git_url(self, get_git_config_val):
        get_git_config_val.return_value = \
            'git://github.com/JNRowe/misc-overlay.git'
        assert_equals(utils.get_repo(), 'JNRowe/misc-overlay')

    @patch('hubugs.utils.get_git_config_val')
    def test_git_url_no_suffix(self, get_git_config_val):
        get_git_config_val.return_value = \
            'git://github.com/JNRowe/misc-overlay'
        assert_equals(utils.get_repo(), 'JNRowe/misc-overlay')

    @patch('hubugs.utils.get_git_config_val')
    def test_https_url(self, get_git_config_val):
        get_git_config_val.return_value = \
            'https://JNRowe@github.com/JNRowe/misc-overlay.git'
        assert_equals(utils.get_repo(), 'JNRowe/misc-overlay')

    @patch('hubugs.utils.get_git_config_val')
    def test_https_url_no_suffix(self, get_git_config_val):
        get_git_config_val.return_value = \
            'https://JNRowe@github.com/JNRowe/misc-overlay'
        assert_equals(utils.get_repo(), 'JNRowe/misc-overlay')

    @patch('hubugs.utils.get_git_config_val')
    def test_http_url(self, get_git_config_val):
        get_git_config_val.return_value = \
            'http://JNRowe@github.com/JNRowe/misc-overlay.git'
        assert_equals(utils.get_repo(), 'JNRowe/misc-overlay')

    @patch('hubugs.utils.get_git_config_val')
    def test_http_url_no_suffix(self, get_git_config_val):
        get_git_config_val.return_value = \
            'http://JNRowe@github.com/JNRowe/misc-overlay'
        assert_equals(utils.get_repo(), 'JNRowe/misc-overlay')

    @patch('hubugs.utils.get_git_config_val')
    def test_http_url_no_auth(self, get_git_config_val):
        get_git_config_val.return_value = \
            'http://github.com/JNRowe/misc-overlay.git'
        assert_equals(utils.get_repo(), 'JNRowe/misc-overlay')

    @patch('hubugs.utils.get_git_config_val')
    def test_http_url_no_suffix_no_auth(self, get_git_config_val):
        get_git_config_val.return_value = \
            'http://github.com/JNRowe/misc-overlay'
        assert_equals(utils.get_repo(), 'JNRowe/misc-overlay')

    @patch('hubugs.utils.get_git_config_val')
    @raises(ValueError)
    def test_broken_url(self, get_git_config_val):
        get_git_config_val.return_value = 'git://github.com/misc-overlay.git'
        utils.get_repo()

    @patch('hubugs.utils.get_git_config_val')
    @raises(ValueError)
    def test_no_url(self, get_git_config_val):
        get_git_config_val.return_value = None
        utils.get_repo()

    @patch('hubugs.utils.get_git_config_val')
    @raises(ValueError)
    def test_invalid_url(self, get_git_config_val):
        get_git_config_val.return_value = 'http://example.com/dog.git'
        utils.get_repo()


@patch('hubugs.utils.check_output')
def test_get_term_size(check_output):
    check_output.return_value = '62 118'
    term = utils.get_term_size()
    assert_equals(term.lines, 62)
    assert_equals(term.columns, 118)


class SetApi(TestCase):
    @patch('hubugs.utils.get_repo')
    def test_no_project(self, get_repo):
        get_repo.return_value = 'JNRowe/misc-overlay'
        namespace = argparse.Namespace(project=None)
        utils.set_api(namespace)
        assert_equals(namespace.project, 'JNRowe/misc-overlay')

    def test_project(self):
        namespace = argparse.Namespace(project='JNRowe/misc-overlay')
        utils.set_api(namespace)
        assert_equals(namespace.project, 'JNRowe/misc-overlay')

    @patch('github2.request.GithubRequest.raw_request')
    def test_api(self, raw_request):
        raw_request.return_value = {u'issues': []}
        namespace = argparse.Namespace(project='JNRowe/misc-overlay')
        utils.set_api(namespace)
        assert_equals(namespace.api('list'), [])



#
# coding=utf-8
"""hubugs - Simple client for GitHub issues"""
# Copyright (C) 2010-2011  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from . import _version


__version__ = _version.dotted
__date__ = _version.date
__author__ = "James Rowe <jnrowe@gmail.com>"
__copyright__ = "Copyright (C) 2010-2011  James Rowe <jnrowe@gmail.com>"
__license__ = "GNU General Public License Version 3"
__credits__ = ""
__history__ = "See git repository"

from email.utils import parseaddr

__doc__ += """.

``hubugs`` is a very simple client for working with `GitHub's issue tracker`_.

.. _GitHub's issue tracker: http://github.com/blog/411-github-issue-tracker

.. moduleauthor:: `%s <mailto:%s>`__
""" % parseaddr(__author__)

import sys

import argh

from . import (template, utils)


COMMANDS = []


def command(func):
    """Simple decorator to add function to ``COMMANDS`` list

    The purpose of this decorator is to make the definition of commands simpler
    by reducing duplication, it is purely a convenience.

    :param func func: Function to wrap
    :rtype: func
    :returns: Original function
    """
    COMMANDS.append(func)
    return func


# Convenience wrappers for defining command arguments
# pylint: disable-msg=C0103
bugs_arg = argh.arg("bugs", nargs="+", type=int,
                    help="bug number(s) to operate on")

message_arg = argh.arg("-m", "--message", help="comment text")

order_arg = argh.arg("-o", "--order", default="number",
                     choices=["number", "updated", "votes"],
                     help="sort order for listing bugs")

states_arg = argh.arg("-s", "--state", default="open",
                      choices=["open", "closed", "all"],
                      help="state of bugs to operate on")

stdin_arg = argh.arg("--stdin", default=False,
                     help="read message from standard input")

title_arg = argh.arg("title", help="title for the new bug", nargs="?")
body_arg = argh.arg("body", help="body for the new bug", nargs="?")
# pylint: enable-msg=C0103


@command
@argh.alias("list")
@states_arg
@argh.arg("-l", "--label", help="list bugs with specified label",
          metavar="label")
@order_arg
def list_bugs(args):
    "listing bugs"
    if args.label:
        bugs = args.api("list_by_label", args.label)
        if not args.state == "all":
            bugs = filter(lambda x: x.state == args.state, bugs)
    else:
        bugs = []
        states = ["open", "closed"] if args.state == "all" else [args.state, ]
        for state in states:
            bugs.extend(args.api("list", state))
    return template.display_bugs(bugs, args.order, state=args.state)


@command
@states_arg
@order_arg
@argh.arg("term", help="term to search bugs for")
def search(args):
    "searching bugs"
    states = ["open", "closed"] if args.state == "all" else [args.state, ]
    bugs = []
    for state in states:
        bugs.extend(args.api("search", args.term, state))
    return template.display_bugs(bugs, args.order, term=args.term,
                                 state=args.state)


@command
@argh.arg("-f", "--full", default=False, help="show bug including comments")
@argh.arg("-p", "--patch", default=False,
          help="display patches for pull requests")
@bugs_arg
def show(args):
    "displaying bugs"
    tmpl = template.get_template('view', '/issue.txt')
    for bug_no in args.bugs:
        try:
            bug = args.api("show", bug_no)
        except RuntimeError as error:
            if "Issue #%s not found" % bug_no in error.args[0]:
                yield utils.fail("Issue %r not found" % bug_no)
                break
            else:
                raise

        if args.full:
            comments = args.api("comments", bug.number)
        else:
            comments = []
        if args.patch and bug.pull_request_url:
            patch = args._http.request(bug.patch_url)[1]
        else:
            patch = None
        yield tmpl.render(bug=bug, comments=comments, full=True, patch=patch)


@command
@argh.alias("open")
@stdin_arg
@title_arg
@body_arg
@argh.wrap_errors(template.EmptyMessageError)
def open_bug(args):
    "opening new bugs"
    if args.stdin:
        text = sys.stdin.readlines()
    elif not args.title:
        text = template.edit_text("open").splitlines()
    if args.stdin or not args.title:
        title = text[0]
        body = "\n".join(text[1:])
    else:
        title = args.title
        body = args.body
    bug = args.api("open", title, body)
    return utils.success("Bug %d opened" % bug.number)


@command
@stdin_arg
@message_arg
@bugs_arg
@argh.wrap_errors(template.EmptyMessageError)
def comment(args):
    "commenting on bugs"
    if args.stdin:
        message = sys.stdin.read()
    elif args.message:
        message = args.message
    else:
        message = template.edit_text()
    for bug in args.bugs:
        try:
            args.api("comment", bug, message)
        except RuntimeError as error:
            if "Issue #%s not found" % bug in error.args[0]:
                yield utils.fail("Issue %r not found" % bug)
            else:
                raise


@command
@stdin_arg
@title_arg
@body_arg
@bugs_arg
@argh.wrap_errors(template.EmptyMessageError)
def edit(args):
    "editing bugs"
    if (args.title or args.stdin) and len(args.bugs) > 1:
        raise argh.CommandError("Can not use --stdin or command line "
                                "title/body with multiple bugs")
    for bug in args.bugs:
        if args.stdin:
            text = sys.stdin.readlines()
        elif not args.title:
            try:
                current = args.api("show", bug)
            except RuntimeError as error:
                if "Issue #%s not found" % bug in error.args[0]:
                    yield utils.fail("Issue %r not found" % bug)
                    continue
                else:
                    raise
            current_data = {"title": current.title, "body": current.body}
            text = template.edit_text("open", current_data).splitlines()
        if args.stdin or not args.title:
            title = text[0]
            body = "\n".join(text[1:])
        else:
            title = args.title
            body = args.body

        try:
            args.api("edit", bug, title, body)
        except RuntimeError as error:
            if "Issue #%s not found" % bug in error.args[0]:
                yield utils.fail("Issue %r not found" % bug)
            else:
                raise


@command
@stdin_arg
@message_arg
@bugs_arg
def close(args):
    "closing bugs"
    if args.stdin:
        message = sys.stdin.read()
    elif not args.message:
        try:
            message = template.edit_text()
        except template.EmptyMessageError:
            # Message isn't required for closing, but it is good practice
            message = None
    else:
        message = args.message
    for bug in args.bugs:
        try:
            if message:
                args.api("comment", bug, message)
            args.api("close", bug)
        except RuntimeError as error:
            if "Issue #%s not found" % bug in error.args[0]:
                yield utils.fail("Issue %r not found" % bug)
            else:
                raise


@command
@stdin_arg
@message_arg
@bugs_arg
def reopen(args):
    "reopening closed bugs"
    if args.stdin:
        message = sys.stdin.read()
    elif not args.message:
        try:
            message = template.edit_text()
        except template.EmptyMessageError:
            # Message isn't required for reopening, but it is good practice
            message = None
    else:
        message = args.message
    for bug in args.bugs:
        try:
            if message:
                args.api("comment", bug, message)
            args.api("reopen", bug)
        except RuntimeError as error:
            if "Issue #%s not found" % bug in error.args[0]:
                yield utils.fail("Issue %r not found" % bug)
            else:
                raise


@command
@argh.arg("-a", "--add", action="append", default=[],
          help="add label to issue", metavar="label")
@argh.arg("-r", "--remove", action="append", default=[],
          help="remove label from issue", metavar="label")
@bugs_arg
def label(args):
    "labelling bugs"
    for bug in args.bugs:
        try:
            for string in args.add:
                args.api("add_label", bug, string)
            for string in args.remove:
                args.api("remove_label", bug, string)
        except RuntimeError as error:
            if "Issue #%s not found" % bug in error.args[0]:
                yield utils.fail("Issue %r not found" % bug)
            else:
                raise


def main():
    """Main script"""
    description = __doc__.splitlines()[0].split("-", 1)[1]
    epilog = "Please report bugs to the JNRowe/hubugs project"
    parser = argh.ArghParser(description=description, epilog=epilog,
                             version="%%(prog)s %s" % __version__)
    parser.add_argument("-p", "--project", action=utils.ProjectAction,
                        help="GitHub project to operate on",
                        metavar="project")
    parser.add_commands(COMMANDS)
    parser.dispatch(pre_call=utils.set_api)

if __name__ == '__main__':
    main()



from collections import namedtuple
from unittest import TestCase

from mock import (Mock, patch)
from nose.tools import (assert_equals, assert_true, raises)
from pygments import (formatters, lexers)

from hubugs import template


class Highlight(TestCase):
    def pyg_side_effect(*args, **kwargs):
        return namedtuple('Call', 'args kwargs')(args, kwargs)

    @patch('hubugs.template.pyg_highlight')
    @patch('sys.stdout')
    def test_no_highlight(self, stdout, pyg_highlight):
        stdout.isatty = Mock(return_value=False)
        pyg_highlight.side_effect = self.pyg_side_effect
        assert_equals(template.highlight('s'), 's')

    @patch('hubugs.template.pyg_highlight')
    @patch('sys.stdout')
    def test_highlight(self, stdout, pyg_highlight):
        stdout.isatty = Mock(return_value=True)
        pyg_highlight.side_effect = self.pyg_side_effect
        result = template.highlight('+++ a\n--- b\n+Test\n')
        assert_true(isinstance(result.args[2], lexers.DiffLexer))
        assert_true(isinstance(result.args[3],
                               formatters.terminal.TerminalFormatter))

    @patch('hubugs.template.pyg_highlight')
    @patch('sys.stdout')
    def test_highlight_lexer(self, stdout, pyg_highlight):
        stdout.isatty = Mock(return_value=True)
        pyg_highlight.side_effect = self.pyg_side_effect
        result = template.highlight('True', 'python')
        assert_true(isinstance(result.args[2], lexers.PythonLexer))

    @patch('hubugs.template.pyg_highlight')
    @patch('sys.stdout')
    def test_highlight_formatter(self, stdout, pyg_highlight):
        stdout.isatty = Mock(return_value=True)
        pyg_highlight.side_effect = self.pyg_side_effect
        result = template.highlight('True', formatter='terminal256')
        assert_true(isinstance(result.args[3],
                               formatters.terminal256.Terminal256Formatter))


class EditText(TestCase):
    @patch('subprocess.check_call')
    @raises(template.EmptyMessageError)
    def test_no_message(self, check_call):
        check_call.return_value = True
        template.edit_text()

    @patch('subprocess.check_call')
    def test_message(self, check_call):
        def side_effect(args):
            open(args[1], 'w').write('Some message')
        check_call.side_effect = side_effect
        assert_equals(template.edit_text(), 'Some message')

    @patch('subprocess.check_call')
    def test_message_comments(self, check_call):
        def side_effect(args):
            open(args[1], 'w').write('Some message\n#Some comment')
        check_call.side_effect = side_effect
        assert_equals(template.edit_text(), 'Some message')

    @patch('subprocess.check_call')
    def test_message_prefill(self, check_call):
        check_call.return_value = True
        assert_equals(template.edit_text('open',
                                         data={'title': 'Some message'}),
                      'Some message')



#
# coding=utf-8
"""utils - Utility functions for hubugs"""
# Copyright (C) 2010-2011  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import inspect
import os
import re
import subprocess
import sys

from collections import namedtuple

import argh
import httplib2

from github2 import core as ghcore
from github2.client import Github

try:
    from termcolor import colored
except ImportError:  # pragma: no cover
    colored = None  # pylint: disable-msg=C0103

# Select colours if terminal is a tty
# pylint: disable-msg=C0103
if colored and sys.stdout.isatty():
    success = lambda s: colored(s, "green")
    fail = lambda s: colored(s, "red")
    warn = lambda s: colored(s, "yellow")
else:  # pragma: no cover
    success = fail = warn = str
# pylint: enable-msg=C0103


# Use timezone aware datetimes in github2 package
ghcore.NAIVE = False


class ProjectAction(argh.utils.argparse.Action):
    """argparse action class for setting project"""
    def __call__(self, parser, namespace, project, option_string=None):
        "Set fully qualified GitHub project name"
        if not "/" in project:
            user = os.getenv("GITHUB_USER", get_git_config_val("github.user"))
            if not user:
                raise parser.error("No GitHub user setting!")
            project = "%s/%s" % (user, project)
        try:
            # Check for project validity early on.  This check is normally less
            # than a 500 bytes transfer
            get_github_api().repos.show(project)
        except RuntimeError as error:
            if "Repository not found" in error.args[0]:
                raise parser.error(fail("Project %r not found" % project))
            else:
                raise
        except httplib2.ServerNotFoundError:
            raise parser.error(fail("Project lookup failed.  Network or "
                                    "GitHub down?"))
        except EnvironmentError as error:
            raise parser.error(error.args[0])

        namespace.project = project


def check_output(args):
    """Simple check_output implementation for Python 2.6 compatibililty

    :param list args: Command and arguments to call
    :rtype: str:
    :return: Command output
    :raise subprocess.CalledProcessError: If command execution fails
    """
    try:
        return subprocess.check_output(args)
    except AttributeError:
        process = subprocess.Popen(args, stdout=subprocess.PIPE)
        output, unused_err = process.communicate()
        retcode = process.poll()
        if retcode:
            raise subprocess.CalledProcessError(retcode, args[0])
        return output


def get_github_api():
    """Create a GitHub API instance

    :rtype: ``Github``
    :return: Authenticated GitHub API instance
    """
    user = os.getenv("GITHUB_USER", get_git_config_val("github.user"))
    token = os.getenv("GITHUB_TOKEN", get_git_config_val("github.token"))
    if not user or not token:
        raise EnvironmentError("No GitHub authentication settings found!")

    if "cache" in inspect.getargspec(Github.__init__).args:
        xdg_cache_dir = os.getenv("XDG_CACHE_HOME",
                                  os.path.join(os.getenv("HOME", "/"),
                                               ".cache"))
        cache_dir = os.path.join(xdg_cache_dir, "hubugs")
        kwargs = {"cache": cache_dir}
    else:
        kwargs = {}
    return Github(username=user, api_token=token, **kwargs)


def get_git_config_val(key, default=None):
    """Fetch a git configuration value

    :param str key: Configuration value to fetch
    """
    try:
        output = check_output(["git", "config", key]).strip()
    except subprocess.CalledProcessError:
        output = default
    return output


def get_editor():
    """Choose a suitable editor

    This follows the method defined in :manpage:`git-var(1)`

    :rtype: ``str``
    :return: Users chosen editor, or ``vi`` if not set"""
    # Match git for editor preferences
    editor = os.getenv("GIT_EDITOR")
    if not editor:
        editor = get_git_config_val("core.editor")
        if not editor:
            editor = os.getenv("VISUAL", os.getenv("EDITOR", "vi"))
    return editor


def get_repo():
    "Extract GitHub project name from config"
    data = get_git_config_val("remote.origin.url")
    if not data:
        raise ValueError("No `origin' remote found")
    match = re.match(r"""
        (?:git(?:@|://)  # SSH or git protocol
          |https?://
           (?:.*@)?)  # HTTP URLs support optional auth data
        github.com[:/]  # hostname, : sep for SSH URL
        ([^/]+/.*?)  # project
        (?:.git)?$  # .git suffix is optional in all clone URLs""",
                     data, re.VERBOSE)
    if match:
        return match.groups()[0]
    else:
        raise ValueError("Unknown project")


def get_term_size():
    """Fetch the current terminal size

    :rtype: ``namedtuple``
    :return: Number of lines and columns in current terminal
    """
    lines, columns = map(int, check_output(["stty", "size"]).split())
    return namedtuple('Tty', 'lines columns')(lines, columns)


def set_api(args):
    """Add authenticated issues API object to args namespace

    :param argparse.Namespace args: argparse namespace to operate on
    """
    if not args.project:
        args.project = get_repo()
    api = get_github_api()
    issues = api.issues

    def api_method(method, *opts, **kwargs):
        "Wrapper for calling functions from api.issues"
        return getattr(issues, method)(args.project, *opts, **kwargs)
    args.api = api_method
    # Include a direct httplib2.Http object, for non-issues related network
    # access.
    args._http = api.request._http
